/// <reference path='types.d.ts' />
var currentRange = require('current-range');
var currentSelection = require('current-selection');
var isBackward = require('selection-is-backward');
var setRange = require('selection-set-range');
/**
 * Abstract `Command` base class to make implementing custom commands easier.
 *
 * @class
 * @public
 */
var AbstractCommand = (function () {
    function AbstractCommand(doc) {
        if (doc === void 0) { doc = document; }
        this.document = doc;
    }
    AbstractCommand.prototype.execute = function (range, value) {
        var hasRange = !!(range && range instanceof Range);
        var backward;
        var selection;
        if (!hasRange) {
            selection = currentSelection(this.document);
            backward = isBackward(selection);
            range = currentRange(selection);
        }
        // if there's no Range at this point, then we can bail
        if (!range)
            return;
        this._execute(range, value);
        if (!hasRange) {
            // when no Range was explicitly passed in then we must reset
            // the document's Selection
            setRange(selection, range, backward);
        }
    };
    AbstractCommand.prototype.queryEnabled = function (range) {
        if (!range)
            range = currentRange(this.document);
        if (!range)
            return false;
        return Boolean(this._queryEnabled(range));
    };
    AbstractCommand.prototype.queryState = function (range) {
        if (!range)
            range = currentRange(this.document);
        if (!range)
            return false;
        return Boolean(this._queryState(range));
    };
    AbstractCommand.prototype._execute = function (range, value) {
        throw new Error('implement _execute()!');
    };
    AbstractCommand.prototype._queryEnabled = function (range) {
        return !!range;
    };
    AbstractCommand.prototype._queryState = function (range) {
        return false;
    };
    return AbstractCommand;
})();
module.exports = AbstractCommand;
